@@ -1,5 +1,5 @@
-#ifndef SANITY_H
-#define SANITY_H
+#ifndef IDENTIFY_H
+#define IDENTIFY_H
 
 #include <stdio.h>
 
@@ -12,12 +12,21 @@
 #define FAT32NT_BR 6
 #define MBR_95B    7
 #define MBR_2000   8
-#define MBR_SYSLINUX   9
-#define MBR_ZERO   10
-#define NUMBER_OF_RECORD_TYPES 	11
+#define MBR_VISTA  9
+#define MBR_WIN7   10
+#define MBR_SYSLINUX  11
+#define MBR_ZERO   12
+#define FAT16FD_BR 13
+#define FAT32FD_BR 14
+#define NTFS_BR    15
+#define MBR_GPT_SYSLINUX  16
+#define FAT32PE_BR 17
+#define NUMBER_OF_RECORD_TYPES 	18
 
 /* Returns the number of sectors on disk before partition */
 unsigned long partition_start_sector(FILE *fp);
+/* Returns the number of heads for the drive of a partition */
+unsigned short partition_number_of_heads(FILE *fp);
 
 /* Returns TRUE if writing a boot record of type iBr seems like a good idea,
    otherwise FALSE */
